Sets
i   demand node   /1*8/
j   candidate location /1*4/
set iter/iter1*iter30/;
;
parameters
f(j)  fixed cost/1 4400,  2  4500, 3   4250, 4   4250/
k(j) capacity   /1 600,  2  700, 3   500, 4   550/
h(i) demand     /1 100, 2  150, 3  175, 4   125, 5  180, 6  140, 7  120, 8  160/
;
table
c(j,i) trasportation cost
   1    2    3    4     5    6    7     8
1  26  13   19   11    17   25   20    21
2  27  19   22   22    15   26   19    16
3  28  21   12   22    13   27   23    23
4  17  19   26   24    15   23   26    26

;
positive variable

z(i,j)
;
binary variable

x(j)
;
free variable
v
;
equations

obj
eq1
eq3
;

obj.. v=e=sum(j,f(j)*x(j))+sum((i,j),c(j,i)*z(i,j)) ;

eq1(i).. sum(j,z(i,j))=e=h(i);
eq3(j)  .. sum(i,z(i,j))=l=k(j)*x(j);

model daskin as /obj,eq1,eq3/;
option optcr=0;
option mip=cplex;
solve daskin minimizing v using mip;
display   v.l, x.l, z.l;

*/////////////////////// Benders Dual Sub Problem ////////////////

x.l(j)=1;
variables
u(i)  'dual variable for allocation constraint'
w(j) 'dual variable for location constraint'
zdsp 'the objective function of DSP'
;
positive variable
w(j);
free variable
zdsp
u(i);

equations
dspobj  'the equation of DSP objective function'
eq4(i,j) 'the DSP constraint'
;

dspobj .. zdsp=e=sum(i,h(i)*u(i))-sum(j,k(j)*x.l(j)*w(j))+sum(j,f(j)*x.l(j));
*z(i,j):
eq4(i,j)..u(i)-w(j)=l=c(j,i);

model dualsubproblem /dspobj, eq4/;

*///////////////////// Benders dual sub problem for unbounded situtions ///////////


equations
extremerayobj      'modified objective function'
extremeraycons     ' modified constraint'
;
extremerayobj .. sum(i,h(i)*u(i))-sum(j,k(j)*x.l(j)*w(j))=e=1;
extremeraycons(i,j) ..  u(i)-w(j)=l=0;
*normalized(i,j) ..  u(i)+w(j)=e=1;

model extremeray /extremerayobj,extremeraycons,dspobj/;


*//////////////////// Benders main problem /////////////////////
set optcut(iter)  'dynamic optimal cut set'
set unbcut(iter) 'dynamic feasibility cut set'
;
optcut(iter)=no;
unbcut(iter)=no;

variables
lb  'the objective function of main problem (lower bound of original problem)'
;

parameter
uu(i,iter)
ww(j,iter);

equations
optimalitycut(iter)  'benders optimality cutting plane'
feasibilitycut(iter) 'benders feasibility cutting plane'
*eq10
;

optimalitycut(optcut) .. lb=g=sum(i,h(i)*uu(i,optcut))-sum(j,k(j)*x(j)*ww(j,optcut))+sum(j,f(j)*x(j));
feasibilitycut(unbcut) .. sum(i,h(i)*uu(i,unbcut))-sum(j,k(j)*x(j)*ww(j,unbcut))=l=0;
*eq10(j,unbcut)  .. sum(i,z(i,j))=l=k(j)*x(j);

*feasibilitycut(unbcut) .. sum(i,uu(i,unbcut))

model master /optimalitycut, feasibilitycut/;
*///////////////////////////// benders algorithm //////////////////////////
parameter results(iter,*);
parameter nonconverged /yes/;
scalar  unbounded  /1.0e9/;
scalar uperbound /+inf/;
scalar lowerbound /-inf/;

loop(iter$(nonconverged),

solve  dualsubproblem maximizing zdsp using lp;
if (zdsp.l<unbounded,
uperbound=zdsp.l;
optcut(iter)=yes;
else
solve  extremeray maximizing zdsp using lp;
unbcut(iter)=yes;
);

uu(i,iter)=u.l(i);
ww(j,iter)=w.l(j);

option optcr=0;
solve master minimizing lb using mip;
display lb.l;

abort$(master.modelstat<>1) "original model not solved optimality";
*abort$(dualsubproblem.modelstat=4)
lowerbound=lb.l;
results(iter,'lb')=lowerbound;
results(iter,'ub')=uperbound;
nonconverged$((uperbound-lowerbound)<0.1)=no;
);

display  uperbound, lowerbound;

execute_unload 'DAS2.gdx'

